% Prepare drive plan from PLATON VisualGrids export file of bus route
% Mapping module  must be loaded before:
pkg load mapping   ;
addpath("c:/Octave/Octave-5.1.0.0/home/ocz/jsonlab-1.9.8") ;


clear;
close all;
wb1=waitbar(0,"Adding elevation data...");

d=0;                               % total distance of route
vmax= 50/3.6; % in m/s

% data import
[Shape,delimiterOut,headerlinesOut] = importdata('373-Cmentarz-Szpital-st.csv',',',1);
[m,n]    = size(Shape.data);       % m  shape points, n columns

Route = zeros (m,12); % x, y, h, R, H, azi, dist, phi, stopid ,radius, d, speed
                      % 1  2  3  4  5  6    7     8    9       10      11 12

m1=0;

% stop_seq  shape_seq       elevation  phi         
% 1         2         3     4          5
stop_shape=[];  
stop_seq=0;  
% populate x, y and h
for k=1:m
      waitbar(k/m,wb1) ;
      Route(k,1) = Shape.data(k,1);
      Route(k,2) = Shape.data(k,2);

      loc = sprintf ("%f,%f", Shape.data(k,1),Shape.data(k,2));
      url = strcat ("http://open.mapquestapi.com/elevation/v1/profile?key=p7kchAO8lK0pyv6USwY7AcBDLRyjGQ82&shapeFormat=raw&latLngCollection=",loc);
      s = urlread (url);
      s_json = loadjson(s);
      h=s_json.elevationProfile{1,1}.height;
      Route(k,3) = h;
      %pause (0.05);
      Route(k,9) = Shape.data(k,3);
      if Route(k,9) > 0
        stop_seq+=1;
        m1+=1;
        stop_shape = [stop_shape; [stop_seq k 0 Route(k,3) 0]];
      end;
end;


% simulate the drive and add up all distances between points, calculate grade, radius and azimuth
for k=1:m-1
  
  lat1   = Route(k, 1);
  lon1   = Route(k, 2);
  elev1  = Route(k, 3);

  lat2   = Route(k+1, 1);
  lon2   = Route(k+1, 2);
  elev2  = Route(k+1, 3);

  grade  = elev2-elev1;
  
  [ret1] = wgs2googlem(lon1, lat1);
  [ret2] = wgs2googlem(lon2, lat2);
  % radius is not accurate because of not equidistant points, but anyway ...
  if k < m-2 
    lat3   = Route(k+2, 1);
    lon3   = Route(k+2, 2);
    elev3  = Route(k+2, 3);
    [ret3] = wgs2googlem(lon3, lat3);
    
    xk =0;
    yk =0;
    xk1=ret2(1)-ret1(1);
    yk1=ret2(2)-ret1(2);
    xk2=ret3(1)-ret1(1);
    yk2=ret3(2)-ret1(2);
    % solve AX=B -> X=A\B
    A=[1 -xk -yk; 1 -xk1 -yk1; 1 -xk2 -yk2];
    B=[-xk^2+yk^2; -xk1^2+yk1^2; -xk2^2+yk2^2];
    X=A\B;
    xm=X(2)/2;
    ym=X(3)/2;
    radius=sqrt(xm^2+ym^2-X(1));
  else 
    radius=10000000;
  end;

  azi    = azimuth (lat1,lon1,lat2,lon2);
  dist   = (sqrt((ret1(1) - ret2(1))^2 + (ret1(2) - ret2(2))^2))/ret1(3); % in meters
  
  if  dist ~= 0 phi = atan(grade/dist);
  else          phi = 0;               end;
  
  Route(k,4)   = ret1(1);
  Route(k,5)   = ret1(2);
  Route(k+1,4) = ret2(1);
  Route(k+1,5) = ret2(2);
  Route(k,6)   = azi;
  Route(k,7)   = dist;
  Route(k,8)   = phi;
  Route(k,10)  = round(radius);
  Route(k,11)  = d;
  Route(k+1,11)= d+dist;

  if dist < 40 drivingspeed = vmax*dist/40;
  else         drivingspeed = vmax; end;
  Route(k,12)= drivingspeed;                 # desired speed is dependent on distance to next shape/fixpoint 
  
  d = d + dist;
end

% populate all trip segment distances from stop to stop
for l=1:m1-1
    stop_shape(l,3) = Route(stop_shape(l+1,2),11) - Route(stop_shape(l,2),11);
    elev1  = stop_shape(l,   4);
    elev2  = stop_shape(l+1, 4);
    grade  = elev2-elev1;
    if  stop_shape(l,3) ~= 0 phi = atan(grade/stop_shape(l,3));
    else                     phi = 0;               end;
    stop_shape(l,5) = phi; 
end;
                          
csvwrite('route.csv',Route);

% plan a second by second driving velocity for the trip
% as output of prepare.m and input to simulation model

v_plan=0;                 % planned velocity at second
a_a=1.02;                 % acceleration
a_b=0.8;                  % deceleration
v_max=14;                 % maximum velocity
t_a=v_max/a_a;            % acceleration time
t_b=v_max/a_b;            % braking time
s=0;
v=0;
for l=1:1:round(t_a)      % Numerical integration 
  v = v + a_a*1;          % Vehicle speed m/s
  s = s + v*1;            % Vehicle travelled distance m
end;
s_a=s;                    % acceleration distance

v=v_max;
s=0;
for l=1:1:round(t_b)      % Numerical integration 
  v = v - a_b*1;          % Vehicle speed m/s
  s = s + v*1;            % Vehicle travelled distance m
end;
s_b=s;                    % braking distance

vplan = [];               % vector of velocities
gplan = [];               % vector of grades

for segment = 1 :1: m1
  s_end = stop_shape(segment,3);  % total distance between stops
  s_coast= s_end-s_a-s_b;         % distance of coast at v_max
  t_coast= s_coast/v_max;         % time of coast at v_max

  sg=s_end;
  v0=sqrt(-2*sg/(-1/a_b+(1/(a_a+a_b))));
  s0=sg+v0^2/(-2*a_b);
  t_c=v0/(a_a+a_b);
  
  if s_a+s_b < s_end    % acc plus brkng dist. are lower that stop dist.
    t_end = t_a+t_coast+t_b;  % total trip time between stops
    for sec=0:1:round(t_a)                   
      v_plan=v_plan+a_a*1; 
      vplan= [vplan v_plan];  % acceleration phase
      gplan= [gplan stop_shape(segment,5)]; % phi
    end; 
    for sec=round(t_a):1:round(t_a+t_coast)  
      v_plan=v_plan+0;     
      vplan= [vplan v_plan];  % coast (sailing) phase
      gplan= [gplan stop_shape(segment,5)]; % phi
    end;
    for sec=round(t_a+t_coast):1:(t_end)             
      v_plan=v_plan-a_b*1; 
      vplan= [vplan v_plan];  % deceleration phase
      gplan= [gplan stop_shape(segment,5)]; % phi
    end;
  else
    t_end= v0/a_b;
    for sec=0:1:round(t_c)   % stop dist. is lower than acc plus brkng. dist.
      v_plan=v_plan+a_a*1; 
      vplan= [vplan v_plan]; % acceleration phase
      gplan= [gplan stop_shape(segment,5)]; % phi
    end; 
    for sec=round(t_c):1:round(t_end)             
      v_plan=v_plan-a_b*1; 
      vplan= [vplan v_plan];  % deceleration phase
      gplan= [gplan stop_shape(segment,5)]; % phi
    end; 
  end;
  
  % add a random dwell time between 30 and 48 seconds for each stop
  for i=1:1:randi([10 15],1,1) 
      vplan= [vplan 0]; 
      gplan= [gplan stop_shape(segment,5)]; % phi
  end;
  v_plan=0;
end;

%%{ 
% To take into account speed reductions at intersections
% for all vplan values lookup in route the referring shape  
% set the vplan value 
%
delta_s = floor(max(Route(:,11)))/size(vplan,2); % m
shapeno=2;
kkk=1;
for lll=1:size(vplan,2);
   spos=lll*delta_s;      % spos is lll equidistant fractions of route length in meters
   % find desired speed in Route(:,10) for the current spos
   if spos > Route(shapeno,11) 
      kkk=shapeno; % yes remember this shape #
      shapeno++;   % go to next shape
   end;  
   if vplan(lll) > Route(kkk,12)
     vplan(lll) = Route(kkk,12); % set vplan to desired speed 
   end
end;
%%}


close (wb1);

figure('Position',[20 60 1500 200]);
plot (vplan);
save ('vplan.mat','vplan'); % rename vplan.mat to vplant.mat / vplanb.mat for use in BusVehicleSimulation
save ('gplan.mat','gplan'); % rename gplan.mat to gplant.mat / gplanb.mat for use in BusVehicleSimulation





